/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveFileCollectionProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Adds output task which provides an archive file of the collection of every
 * compiled files used by the compiled component.
 */
public class ArchiveOutputCompiler extends AbstractOutputCompiler {

  // --------------------------------------------------------------------------
  // Client interfaces.
  // --------------------------------------------------------------------------

  /**
   * The name of the {@link LinkerBuilder archive builder} client interface used
   * by this compiler.
   * 
   * @see #archiveBuilderItf
   */
  public static final String ARCHIVE_BUILDER_ITF = "archive-builder";

  /** The {@link LinkerBuilder archive builder} used by this compiler. */
  public LinkerBuilder       archiveBuilderItf;

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {

    try {
      tasks.getTask("output", container);
      return;
    } catch (final NoSuchElementException e) {
      // continue
    }

    final ArchiveTask outputTask = new ArchiveTask(archiveBuilderItf);
    tasks.addTask("output", container, outputTask);
    outputTask.setFactoryProviderTask(tasks.getTaskHole("type", container));

    final Collection<TaskMap.TaskHole> compilationTasks = getComponentCompilationTasks(
        container, tasks, context);
    for (final TaskMap.TaskHole taskHole : compilationTasks) {
      outputTask.addFileProviderTask(taskHole);
    }
    addSubCompDependancies(container, tasks, outputTask);
  }

  // --------------------------------------------------------------------------
  // Task classes
  // --------------------------------------------------------------------------

  protected static class ArchiveTask
      extends
        AbstractRecursiveFileCollectionProviderTask {

    private final LinkerBuilder builder;

    /**
     * @param builder
     */
    public ArchiveTask(final LinkerBuilder builder) {
      this.builder = builder;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object factory = getFactoryProviderTask().getFactory();
      final File outDir = (File) context.get("objBuildDirectory");
      final File destFile = new File(outDir, factory.toString().replace('.',
          '_')
          + ".a");

      builder.link(destFile, getProvidedFiles().toArray(new File[0]), null,
          context);
      final Collection<File> c = new ArrayList<File>();
      c.add(destFile);
      setResult(c);
    }
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  @Override
  public String[] listFc() {
    final String[] superList = super.listFc();
    final String[] list = new String[superList.length + 1];
    System.arraycopy(superList, 0, list, 0, superList.length);
    list[superList.length] = ARCHIVE_BUILDER_ITF;
    return list;
  }

  @Override
  public void bindFc(final String itf, final Object value)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(ARCHIVE_BUILDER_ITF)) {
      this.archiveBuilderItf = (LinkerBuilder) value;
    } else {
      super.bindFc(itf, value);
    }
  }

  @Override
  public Object lookupFc(final String itf) throws NoSuchInterfaceException {
    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(ARCHIVE_BUILDER_ITF)) {
      return this.archiveBuilderItf;
    } else {
      return super.lookupFc(itf);
    }
  }

  @Override
  public void unbindFc(final String itf) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {
    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(ARCHIVE_BUILDER_ITF)) {
      this.archiveBuilderItf = null;
    } else {
      super.unbindFc(itf);
    }
  }
}
