/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.cecilia.adl.components.AbstractPrimitiveCompilerPluginUser;
import org.objectweb.fractal.cecilia.adl.plugin.PluginException;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Primitive Compiler selector that chooses the compiler to execute based on the
 * {@link OutputFormat output description} of the component. For a given
 * component, the name of the plugin used for compilation is :
 * <code>"&lt;{@link #getADLPackage() package}&gt;.&lt;{@link OutputFormat#getFormat() outputFormat}&gt;&lt;{@link #getADLSuffix() ADLSuffix}&gt;"</code>
 * (Note : in order to respect naming convention, the first char of
 * {@link OutputFormat#getFormat() outputFormat} is transformed to uppercase).
 * 
 * @see org.objectweb.fractal.cecilia.adl.plugin.AbstractPluginUser#getPlugin(Object, Map,
 *      Class)
 */
public class OutputCompilerSelector extends AbstractPrimitiveCompilerPluginUser
    implements
      PrimitiveCompiler,
      OutputCompilerSelectorAttributes {

  /** The executable output format. */
  public static final String EXECUTABLE_FORMAT = "executable";

  // --------------------------------------------------------------------------
  // Attribute fields
  // --------------------------------------------------------------------------
  private String             defaultFormat;
  private String             defaultTopLevelFormat;

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    OutputFormat outputFormat = null;
    String format = null;
    if (container instanceof OutputFormatContainer) {
      outputFormat = ((OutputFormatContainer) container).getOutput();
      if (outputFormat != null) format = outputFormat.getFormat();
    }
    if (format == null) {
      if (path.size() == 0)
        format = defaultTopLevelFormat;
      else
        format = defaultFormat;
    }
    if (EXECUTABLE_FORMAT.equals(format)) {
      format = (String) context.get("executable-format");
    }

    final String pluginName = format.substring(0, 1).toUpperCase()
        + format.substring(1);
    PrimitiveCompiler compiler;
    try {
      compiler = getPrimitiveCompilerPlugin(pluginName, context);
    } catch (final PluginException e) {
      throw new ADLException(
          "An error occurs while loading plugin for output: '" + format + "'",
          (Node) outputFormat, e);
    }

    compiler.compile(path, container, tasks, context);
  }

  // --------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // --------------------------------------------------------------------------

  public String getDefaultFormat() {
    return defaultFormat;
  }

  public void setDefaultFormat(final String defaultFormat) {
    this.defaultFormat = defaultFormat;
  }

  public String getDefaultTopLevelFormat() {
    return defaultTopLevelFormat;
  }

  public void setDefaultTopLevelFormat(final String defaultTopLevelFormat) {
    this.defaultTopLevelFormat = defaultTopLevelFormat;
  }

}
