/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.components;

import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.task.codegen.api.RequireFactoryProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireInstanceProviderTask;
import org.objectweb.fractal.task.core.Task;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Abstract {@link PrimitiveCompiler} which implements the
 * {@link CompilerAttributes} attribute controller interface.
 */
public abstract class AbstractPrimitiveCompiler
    implements
      CompilerAttributes,
      PrimitiveCompiler {

  /**
   * Compiler role enumeration defines names suitable for the
   * {@link CompilerAttributes#getRole() role} of the compiler.
   */
  public static enum CompilerRole {

    /**
     * This role is used when the compiler creates tasks relative to the type
     * definition of a component.
     */
    definition,

    /**
     * This role is used when the compiler creates tasks relative to the
     * instance of a component.
     */
    instance,

    /**
     * This role is used when the compiler creates tasks relative to the factory
     * of a component.
     */
    factory;
  };

  private CompilerRole compilerRole;

  protected CompilerRole getCompilerRole() {
    return compilerRole;
  }

  protected TaskMap.TaskHole addTask(final String taskName, final Task task,
      final Object container, final TaskMap tasks) {
    TaskMap.TaskHole taskHole = tasks.getTaskHole(getFullTaskName(taskName),
        container);
    if (taskHole.isFilled()) return taskHole;

    taskHole = tasks.addTask(getFullTaskName(taskName), container, task);
    if (task instanceof RequireFactoryProviderTask) {
      ((RequireFactoryProviderTask) task).setFactoryProviderTask(tasks
          .getTaskHole("type", container));
    }
    if (compilerRole == CompilerRole.instance
        && (task instanceof RequireInstanceProviderTask)) {
      ((RequireInstanceProviderTask) task).setInstanceProviderTask(tasks
          .getTaskHole("instance", container));
    }
    return taskHole;
  }

  /**
   * Get the {@link TaskMap.TaskHole} having as type <code>taskName</code>.{@link #compilerRole}
   * and <code>container</code> as identifier.
   * 
   * @param taskName
   * @param container
   * @param tasks the TaskMap
   * @return the TaskHole
   */
  protected TaskMap.TaskHole getTaskHole(final String taskName,
      final Object container, final TaskMap tasks) {
    return tasks.getTaskHole(getFullTaskName(taskName), container);
  }

  /**
   * @param taskName
   * @return a String made of the <code>taskName</code> parameter appending "<code>.</code>"
   *         followed by the {@link #compilerRole} of this component.
   */
  protected final String getFullTaskName(final String taskName) {
    return new StringBuilder(taskName).append('.').append(compilerRole)
        .toString();
  }

  // --------------------------------------------------------------------------
  // Implementation of the CompilerAttributes interface
  // --------------------------------------------------------------------------

  public String getRole() {
    return compilerRole.name();
  }

  public void setRole(final String role) {
    compilerRole = CompilerRole.valueOf(role);
  }

}
