/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.components;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.ControllerContainer;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveSourceCodeProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Adds a task that creates a component.
 */
public class PrimitiveComponentCompiler extends AbstractPrimitiveCompiler
    implements
      BindingController {

  // --------------------------------------------------------------------------
  // Client interface.
  // --------------------------------------------------------------------------

  /**
   * The name of the {@link ComponentBuilder} client interface used by this
   * compiler.
   */
  public static final String BUILDER_ITF = "builder";

  /** The {@link ComponentBuilder} used by this compiler. */
  protected ComponentBuilder builderItf;

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {

    String controller = null;
    if (container instanceof ControllerContainer) {
      final ControllerContainer cc = (ControllerContainer) container;
      if (cc.getController() != null) {
        controller = cc.getController().getDescriptor();
      }
    }

    String implementation = null;
    if (container instanceof ImplementationContainer) {
      final Implementation i = ((ImplementationContainer) container)
          .getImplementation();
      if (i != null) {
        implementation = i.getClassName();
      }
    }

    createTask(container, tasks, controller, implementation);
  }

  protected void createTask(final ComponentContainer container,
      final TaskMap tasks, final String controller, final String implementation) {
    final CreateTask task = new CreateTask(builderItf, controller,
        implementation);
    addTask("component", task, container, tasks);
  }

  // --------------------------------------------------------------------------
  // Task class
  // --------------------------------------------------------------------------

  protected static class CreateTask
      extends
        AbstractRecursiveSourceCodeProviderTask {
    protected final ComponentBuilder builder;
    protected final Object           controllerDesc;
    protected final Object           contentDesc;

    /**
     * @param builder
     * @param controllerDesc
     * @param contentDesc
     */
    public CreateTask(final ComponentBuilder builder,
        final Object controllerDesc, final Object contentDesc) {
      this.builder = builder;
      this.controllerDesc = controllerDesc;
      this.contentDesc = contentDesc;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object factory = (getFactoryProviderTask() == null)
          ? null
          : getFactoryProviderTask().getFactory();
      final Object instance = (getInstanceProviderTask() == null)
          ? null
          : getInstanceProviderTask().getInstance();

      setResult(builder.createComponent(factory, instance, controllerDesc,
          contentDesc, getCodePieces(), context));
    }
  }

  public void bindFc(final String itfName, final Object value)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itfName.equals(BUILDER_ITF)) {
      this.builderItf = (ComponentBuilder) value;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + itfName + "'");
    }
  }

  public String[] listFc() {
    return new String[]{BUILDER_ITF};
  }

  public void unbindFc(final String itfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itfName.equals(BUILDER_ITF)) {
      this.builderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + itfName + "'");
    }
  }

  public Object lookupFc(final String itfName) throws NoSuchInterfaceException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itfName.equals(BUILDER_ITF)) {
      return this.builderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + itfName + "'");
    }
  }

}
