/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.idl;

import static org.objectweb.fractal.cecilia.adl.idl.util.CUtil.buildInclude;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Method;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the VFT definitions for interfaces in C. It extends the Abstract
 * Interface visitor and implements the required visit methods.
 */
public class VFTTemplateBuilder extends AbstractInterfaceVisitor
    implements
      IDLBuilder {

  private String itfName;
  private CodeWriter header, footer, methodDefinition, vft = null;

  // --------------------------------------------------------------------------
  // Implementation of the IDLBuilder interface
  // --------------------------------------------------------------------------

  public Object createIDLDefinition(final Object componentType,
      final Object componentInstance, final IDLDefinition idlDefinition,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {
    final CodeWriter cw = new CodeWriter("C VFT definition builder");
    // Pre-visit operations
    itfName = idlDefinition.getName();
    header = new CodeWriter();
    footer = new CodeWriter();
    methodDefinition = new CodeWriter();
    vft = new CodeWriter();
    // Visit
    visit(idlDefinition);
    // Post-visit operations
    cw.append(header.toString());
    cw.append(methodDefinition.toString());
    cw.append(vft.toString());
    cw.append(footer.toString());

    final File file = new File(context.get("idlBuildDirectory").toString(),
        idlDefinition.getName().replace('.', File.separatorChar)
            + "VFTTemplate.h");
    if (!file.getParentFile().exists()) file.getParentFile().mkdirs();
    final PrintStream out = new PrintStream(new FileOutputStream(file));
    out.print(cw.toString());
    out.close();

    return cw.toString();
  }

  // --------------------------------------------------------------------------
  // Implementation of the visitor methods
  // --------------------------------------------------------------------------

  public void enterInterface(final InterfaceDefinition itf) throws Exception {
    header.appendln("/* This file is generated automatically */");
    final String cUpItfName = itfName.toUpperCase().replace('.', '_');
    header.append("#ifndef _").append(cUpItfName).append("_VFT_TEMPLATE_H_")
        .endl();
    header.append("#define _").append(cUpItfName).append("_VFT_TEMPLATE_H_")
        .endl();
    header.appendln(buildInclude(itfName));

    footer.appendln("#endif");

    methodDefinition.append("// The following macro can be used to define ")
        .append("prototypes of the methods that a given component must ")
        .append("implements.").endl();
    methodDefinition.append("#define ").append(cUpItfName).append(
        "_METHOD_DEFINITION(comp, itf) \\").endl();

    vft.append("// The following macro can be used to declare and initialize ")
        .append("a VTF instance for a given component interface.").endl();
    vft.append("#define ").append(cUpItfName).append(
        "_VFT_DECLARATION(comp, itf) \\").endl();
    vft.append("  static struct M").append(itfName.replace('.', '_')).append(
        " comp##_##itf##meth = {\\").endl();
  }

  public void leaveInterface(final InterfaceDefinition itf) throws Exception {
    vft.appendln("};");
  }

  public void visitMethod(final Method method) throws Exception {
    // Write VFT initialization for this method
    final String methodName = "comp##_##itf##_##" + method.getName()
        + "##_method";

    vft.append(methodName).append(", \\").endl();

    // Write method definition
    methodDefinition.append(buildMethodDefinition(method, methodName)).append(
        "; \\").endl();
  }

  public void visitField(final Field field) throws Exception {
    // Nothing to do ...
  }

  // --------------------------------------------------------------------------
  // Utility methods
  // --------------------------------------------------------------------------

  protected String buildMethodDefinition(final Method method,
      final String methodName) {
    return CUtil.buildMethodDefinition(method, false, methodName);
  }
}
