/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.interfaces;

import static org.objectweb.fractal.cecilia.adl.interfaces.InterfaceDecorationUtil.getBoundTo;
import static org.objectweb.fractal.cecilia.adl.interfaces.InterfaceDecorationUtil.getContainer;

import java.util.Map;

import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.task.codegen.api.FactoryProviderTask;
import org.objectweb.fractal.task.codegen.api.InstanceProviderTask;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveSourceCodeProviderTask;
import org.objectweb.fractal.task.core.TaskMap;
import org.objectweb.fractal.task.core.TaskMap.TaskHole;

/**
 * Creates a task for the initialisation of the imported interfaces of a
 * component. If the interface {@link Node node} contains a
 * <code>"bound-to"</code> {@link Node#astGetDecoration(String) decoration},
 * the task will depends on the {@link FactoryProviderTask} and
 * {@link InstanceProviderTask} of the server component. In this case, the
 * builder may statically bind the client interface using
 * {@link Node#astGetDecoration(String) decorations} <code>"bound-to"</code>,
 * <code>"server-type"</code> and <code>"server-instance"</code>.
 * 
 * @see org.objectweb.fractal.cecilia.adl.bindings.BindingResolverLoader
 */
public class ImportedInterfaceInstanceCompiler
    extends
      ImportedInterfaceCompiler {

  @Override
  protected TaskHole createTask(final String taskName,
      final TypeInterface[] interfaces, final ComponentContainer container,
      final TaskMap tasks) {
    String contentName = null;
    if (container instanceof ImplementationContainer) {
      final Implementation impl = ((ImplementationContainer) container)
          .getImplementation();
      if (impl != null) contentName = impl.getClassName();
    }

    final TaskMap.TaskHole[] serverTypeProviders = new TaskMap.TaskHole[interfaces.length];
    final TaskMap.TaskHole[] serverInstanceProviders = new TaskMap.TaskHole[interfaces.length];
    for (int i = 0; i < interfaces.length; i++) {
      final Interface serverItf = getBoundTo(interfaces[i]);
      if (serverItf == null) continue;

      final Object serverComponent = getContainer(serverItf);
      serverTypeProviders[i] = tasks.getTaskHole("type", serverComponent);
      serverInstanceProviders[i] = tasks.getTaskHole("instance",
          serverComponent);
    }

    final InterfaceTask task = new InterfaceTask(builderItf, contentName,
        interfaces, serverTypeProviders, serverInstanceProviders);

    return addTask(taskName, task, container, tasks);
  }

  // --------------------------------------------------------------------------
  // Task class
  // --------------------------------------------------------------------------

  protected static class InterfaceTask
      extends
        AbstractRecursiveSourceCodeProviderTask {
    private InterfaceBuilder         builder     = null;
    private String                   contentType = null;
    private TypeInterface[]          interfaces  = null;
    private final TaskMap.TaskHole[] serverTypeProviders;
    private final TaskMap.TaskHole[] serverInstanceProviders;

    /**
     * @param builder
     * @param contentType
     * @param interfaces
     * @param serverTypeProviders
     * @param serverInstanceProviders
     */
    public InterfaceTask(final InterfaceBuilder builder,
        final String contentType, final TypeInterface[] interfaces,
        final TaskMap.TaskHole[] serverTypeProviders,
        final TaskMap.TaskHole[] serverInstanceProviders) {
      this.builder = builder;
      this.contentType = contentType;
      this.interfaces = interfaces;
      this.serverTypeProviders = serverTypeProviders;
      this.serverInstanceProviders = serverInstanceProviders;

      for (int i = 0; i < serverTypeProviders.length; i++) {
        if (serverTypeProviders[i] == null) continue;
        addPreviousTask(serverTypeProviders[i]);
        addPreviousTask(serverInstanceProviders[i]);
      }
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object factory = getFactoryProviderTask().getFactory();
      final Object instance = getInstanceProviderTask().getInstance();

      for (int i = 0; i < interfaces.length; i++) {
        final Node itf = (Node) interfaces[i];
        if (serverTypeProviders[i] == null) continue;

        itf.astSetDecoration("server-type",
            ((FactoryProviderTask) serverTypeProviders[i].getTask())
                .getFactory());

        itf.astSetDecoration("server-instance",
            ((InstanceProviderTask) serverInstanceProviders[i].getTask())
                .getInstance());
      }

      setResult(builder.createInterfaces(factory, instance, contentType,
          interfaces, getCodePieces(), context));
    }
  }

}
