/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 * Contributor: Alessio Pace
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.cecilia.adl.attributes.AttributeBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds a part of the implementation of the <code>factory</code> interface
 * related to the initialization of the attribute controller structure of the
 * component. <br>
 * This builder produces three code pieces:
 * <ul>
 * <li><code>"instantiate.vardecl"</code> containing the definition of the
 * variables used in the <code>instantiate</code> method.</li>
 * <li><code>"instantiate.implem"</code> containing the actual part of the
 * implementation of the <code>instantiate</code> method.</li>
 * <li><code>"destroy.implem"</code> containing the part of the
 * implementation of the <code>destroy</code> method.</li>
 * </ul>
 * In the <code>instantiate</code> method implementation, the builder
 * generates the initialization of a variable whose name is
 * <code>newComp_attributes</code>. It supposes that the original component
 * is pointed by a variable whose name is <code>originalComp</code> and the
 * allocator interface is designated by a variable whose name is
 * <code>alloc_itf</code>.
 * 
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.components.FactoryImplementationBuilder
 */
public class ACFactoryBuilder implements AttributeBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the AttributesBuilder interface
  // --------------------------------------------------------------------------

  public Object setAttributes(final Object componentType,
      final Object componentInstance, final Object attributeAST,
      final Attribute[] fields, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final String definitionName = componentType.toString().replace('.', '_');
    final Map<String, String> producedCode = new HashMap<String, String>();

    // instantiate method implementation ---------------------------------------
    CodeWriter cw = new CodeWriter(
        "Attribute Controller Factory Builder instantiate variables declaration");
    cw.append("struct ").append(definitionName).append(
        "_attributedskeleton * original_attributes;").endl();
    cw.append("struct ").append(definitionName).append(
        "_attributedskeleton * newComp_attributes;").endl();
    producedCode.put("instantiate.vardecl", cw.toString());

    cw = new CodeWriter(
        "Attribute Controller Factory Builder instantiate implementation");
    cw.append("original_attributes = ").append("(struct ").append(
        definitionName).append("_attributedskeleton *) ").append(
        "originalComp->type.exported.attribute_controller.selfdata;").endl();
    cw.append("newComp_attributes = ").append("(struct ")
        .append(definitionName).append("_attributedskeleton *) ").append(
            "CALL(alloc_itf, alloc, sizeof(struct ").append(definitionName)
        .append("_attributedskeleton));").endl().endl();
    cw.append("memcpy(newComp_attributes, original_attributes, sizeof(struct ")
        .append(definitionName).append("_attributedskeleton));").endl();

    for (int i = 0; i < fields.length; i++) {
      cw.append("newComp_attributes->attributes[").append(i).append(
          "].value_" + fields[i].getName() + "= &(newComp->attributes.")
          .append(fields[i].getName()).append(");").endl();
    }
    producedCode.put("instantiate.implem", cw.toString());

    // destroy method implementation -------------------------------------------
    cw = new CodeWriter(
        "Attribute Controller Factory Builder destroy variable definition");
    cw.append("CALL(alloc_itf, free, ").append(
        "originalComp->type.exported.attribute_controller.selfdata);");
    producedCode.put("destroy.implem", cw.toString());

    return producedCode;
  }
}
