/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ComponentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds a part of the implementation of the <code>factory</code> interface
 * related to the controller structure of the component by aggregating the code
 * produced by the different controller builders. <br>
 * This builder produces four code pieces:
 * <ul>
 * <li><code>"instantiate.vardecl"</code> containing the declaration of the
 * variables used in the <code>instantiate</code> method.</li>
 * <li><code>"instantiate.implem"</code> containing the actual part
 * of the implementation of the <code>instantiate</code> method.</li>
 * <li><code>"definition.vardecl"</code> containing the declaration of the
 * variables used in the <code>instantiate</code> method.</li>
 * <li><code>"destroy.implem"</code> containing the part of the
 * implementation of the <code>destroy</code> method.</li>
 * </ul>
 * 
 * @see ACFactoryBuilder
 * @see BCPrimitiveFactoryBuilder
 * @see CIFactoryBuilder
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.components.FactoryImplementationBuilder
 */
public class ControllerFactoryBuilder implements ComponentBuilder {

  public Object createComponent(final Object componentType,
      final Object componentInstance, final Object controllerDesc,
      final Object contentDesc, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {

    final CodeWriter instantiateDecl = new CodeWriter(
        "Controller Factory Builder instantiate variables declaration");
    final CodeWriter instantiateImplem = new CodeWriter(
        "Controller Factory Builder instantiate implementation");
    final CodeWriter destroyDecl = new CodeWriter(
        "Controller Factory Builder destroy variables declaration");
    final CodeWriter destroyImpl = new CodeWriter(
        "Controller Factory Builder destroy implmentation");

    for (Map.Entry<String, Object> codePiece : codePieces.entrySet()) {
      if (codePiece.getKey().endsWith("instantiate.vardecl")) {
        instantiateDecl.append(codePiece.getValue().toString());

      } else if (codePiece.getKey().endsWith("instantiate.implem")) {
        instantiateImplem.append(codePiece.getValue().toString());

      } else if (codePiece.getKey().endsWith("destroy.vardecl")) {
        destroyDecl.append(codePiece.getValue().toString());

      } else if (codePiece.getKey().endsWith("destroy.implem")) {
        destroyImpl.append(codePiece.getValue().toString());
      } else {
        System.err.println("Unknwon pieces of code name=" + codePiece.getKey());
      }
    }

    final Map<String, String> producedCode = new HashMap<String, String>();
    producedCode.put("instantiate.vardecl", instantiateDecl.toString());
    producedCode.put("instantiate.implem", instantiateImplem.toString());
    producedCode.put("destroy.vardecl", destroyDecl.toString());
    producedCode.put("destroy.implem", destroyImpl.toString());
    return producedCode;
  }

}
