/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.source;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.source.AbstractSourceFileBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Creates C-header source file that contains the definition of the component
 * structure. This builder is applicable for multi-module primitive component
 * written in <code>ThinkMC</code>.
 */
public class DefinitionSourceFileBuilder extends AbstractSourceFileBuilder {

  private static final String FILE_NAME_SUFFIX = ".adl-def.h";

  /**
   * Returns the name of the header file that contains the definition of the
   * component of the given type.
   * 
   * @param componentType a component type.
   * @return name of the file that contains the definition of the component of
   *         the given type.
   */
  public static String getDefinitionSourceFileName(final Object componentType) {
    return componentType.toString().replace('.', File.separatorChar)
        + FILE_NAME_SUFFIX;
  }

  // --------------------------------------------------------------------------
  // Implementation of abstract methods.
  // --------------------------------------------------------------------------

  /*
   * Delegates to #getDefinitionSourceFileName
   */
  @Override
  protected String getFileName(final Object componentType,
      final Object componentInstance, final Map<Object, Object> context) {
    return getDefinitionSourceFileName(componentType);
  }

  /*
   * The content of the source file built by this builder is simply the
   * "definition" code piece.
   */
  @Override
  protected String getFileContent(final Object componentType,
      final Object componentInstance, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter("Definition of component "
        + componentType);

    final String def = (String) codePieces.get("definition");
    if (def == null)
      throw new IllegalArgumentException(
          "Cannot find the \"definition\" code piece.");
    cw.appendln(def);

    final String dataDef = (String) codePieces.get("data-definition");
    if (dataDef == null)
      throw new IllegalArgumentException(
          "Cannot find the \"data-definition\" code piece.");
    cw.appendln(dataDef);

    return cw.toString();
  }
}
